#
# compat.rb -- cross platform compatibility
#
# Author: IPR -- Internet Programming with Ruby -- writers
# Copyright (C) 2002 GOTOU Yuuzou
#
# $IPR: compat.rb,v 1.1 2002/02/05 20:19:45 gotoyuzo Exp $

module Errno
  class ECONNRESET   < SystemCallError; end
  class ECONNABORTED < SystemCallError; end
end
